#!/usr/bin/env python3
import seal.seal
import os
import argparse
import glob
import pkg_resources


def insert_suffix(path, suffix):
    base, ext = os.path.splitext(os.path.basename(path))
    directory = os.path.dirname(path)
    return directory + ("" if len(directory) ==
                        0 else "/") + base + suffix + ext


def main():
    default_logo = "watermark/qed-logo.png"

    parser = argparse.ArgumentParser()
    parser.add_argument("filename", type=str,
                        help="input image filename")
    parser.add_argument("-s", "--suffix", type=str,
                        help="suffix for output image", default="_qed")
    parser.add_argument("-i", "--inverse", help="invert logo",
                        action="store_true")
    parser.add_argument("--byline", help="byline logo", action="store_true")
    parser.add_argument(
        "--monochrome", help="one color logo", action="store_true")
    parser.add_argument("--rectangular", help="use rectangular logo",
                        action="store_true")
    parser.add_argument("-o", "--output", type=str, help="output file name")
    parser.add_argument("--logo", type=str,
                        help="logo file name", default=default_logo)
    parser.add_argument("--position", type=str,
                        help="logo position (LR, LL, UR, UL)", default="LR")
    parser.add_argument("--opacity", type=float,
                        help="logo opacity (0.0 - 1.0)", default=1.0)
    parser.add_argument("--padding", type=int, help="logo's distance from picture's edge",
                        default=50)
    parser.add_argument("--filter", type=str,
                        help="logo filter (positive, negative, dark, white)", default="positive")
    args = parser.parse_args()

    logo = args.logo

    if logo == default_logo:
        if not args.rectangular:
            logo = insert_suffix(logo, "-square")

        if args.byline:
            logo = insert_suffix(logo, "-byline")

        if args.monochrome:
            logo = insert_suffix(logo, "-1c")

        if args.inverse:
            logo = insert_suffix(logo, "-rev")
        
        logo = pkg_resources.resource_filename(__name__, logo)

    logos_dict = {args.position: logo}

    sealer = seal.seal.Seal()

    for filename in glob.glob(os.path.expandvars(os.path.expanduser(args.filename))):
        print(insert_suffix(filename, args.suffix))
        sealer.add_logos(filename, args.output if args.output else insert_suffix(
            filename, args.suffix), logos_dict, args.opacity, args.filter, args.padding)



#!/usr/bin/env python3
from PIL import Image


class Seal:

    def __init__(self):
        return

    def invert(self, color):
        return [(-i + 255) % 255 for i in color]

    def greyscale(self, color):
        return [int(color[0] * 299 / 1000 + color[1] * 587 / 1000 + color[2] * 114 / 1000) for i in range(4)]

    def add_logos(self, main_fname, out_fname, logos_dict, opacity=1.0, filter="positive", padding=50):
        """
        Add logos to an image.
        Logos are expected to be in PNG format to support transparency masks.
        """
        if len(set(logos_dict.keys()) - set(["LR", "LL", "UR", "UL"])) > 0:
            raise Exception(
                "Only LR, LL, UR, UL are supported as keys to the logos dict.")
        if any(map(lambda x: not x.endswith('png'), map(lambda y: y.lower(), logos_dict.values()))):
            raise Exception("Only PNG files are supported.")
        if filter not in ["positive", "negative", "black", "white"]:
            raise Exception(
                "Only positive, negative, black, white filters are available.")

        main = Image.open(main_fname)
        main_w, main_h = main.size

        for pos, logo_fname in logos_dict.items():
            logo = Image.open(logo_fname)

            # height is resized to a fraction of the main image's height
            ratio = 0.15
            logo_w, logo_h = logo.size
            logo_h_new = int(ratio * main_h)
            logo_w_new = int((float(logo_h_new) / logo_h) * logo_w)
            logo = logo.resize((logo_w_new, logo_h_new))
            if logo_h_new > logo_h:
                print("\tWarning: resized logo height to {} beyond original height of {}".format(
                    logo_h_new, logo_h))
            if logo_w_new > logo_w:
                print("\tWarning: resized logo width to {} beyond original width of {}".format(
                    logo_w_new, logo_w))

            # superimpose logo
            if pos == "LR":
                offset = (main_w - logo_w_new - padding,
                          main_h - logo_h_new - padding)
            elif pos == "LL":
                offset = (padding, main_h - logo_h_new - padding)
            elif pos == "UL":
                offset = (padding, padding)
            elif pos == "UR":
                offset = (main_w - logo_w_new - padding, padding)
            logo = Image.blend(Image.new(logo.mode, logo.size), logo, opacity)

            for x in range(logo_w_new):
                for y in range(logo_h_new):
                    color = list(logo.getpixel((x, y)))
                    alpha = color[3]
                    if alpha != 0:
                        if filter == "negative":
                            color = self.invert(color)
                        elif filter == "white":
                            color = self.greyscale(color)
                        elif filter == "black":
                            color = self.invert(self.greyscale(color))
                    logo.putpixel(
                        (x, y), (color[0], color[1], color[2], alpha))
            main.paste(logo, offset, logo)

        main.save(out_fname)



from setuptools import setup, find_packages

setup(
    name = "qed-seal",
    version = "0.2",
    description = "image watermarking tool",
    url = "https://github.com/qed-software/seal",
    author = "QED",
    author_email = "software@qed.ai",
    license = "MIT",
    classifiers = [
        "Programming Language :: Python :: 3"
    ],
    keywords = "watermark seal qed",
    install_requires = ["Pillow", "wheel"],
    packages = find_packages(),
    python_requires = ">=3",
    package_data = {
        "seal": ["watermark/*.png"]
    },
    entry_points = {
        "console_scripts": [
            "seal = seal.watermark:main"
        ]
    }
)
